<?php

// Copyright (C) 2020-2021 by Paweł Pilarczyk, http://www.pawelpilarczyk.com/
//
// This file is part of my scientific software.  This is free software;
// you can redistribute it and/or modify it under the terms of the GNU Affero
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public
// License along with this software; see the file "license.txt".
// If not, see <http://www.gnu.org/licenses/>.

// Started on January 6, 2020. Last revision: April 4, 2021.


// read specific configuration data: name of the program to run
require ("config.php");

// read specific data for the form and the arguments of the program
require ("data.php");


// Generates HTML code for the beginning of a HTML page.
// $title - the title to be put in the header
// $headcode - additional tags or code to be added to the "head" section
// $bodycode - options to add to the "body" tag
// $indexing - should indexing of the page be allowed for the robots?
function html_begin ($title, $headcode, $bodycode, $indexing)
{
	// prepare a space after the word "body" in the <body> tag
	if (empty ($bodycode))
		$bodyspace = "";
	else
		$bodyspace = " ";

	// prepare a tag to keep indexing robots out
	if ($indexing)
		$norobots = "";
	else
		$norobots = "<meta name=\"robots\" " .
			"content=\"noindex,nofollow,noarchive\">\n";

	// return the HTML code for the beginning of the HTML page
	return <<< HTML
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
<meta name="author" content="Pawe&#x142; Pilarczyk">
${norobots}<title>${title}</title>
<link rel="stylesheet" media="screen" type="text/css" href="stylescr.css" />
<link rel="stylesheet" media="print" type="text/css" href="styleprt.css" />
${headcode}</head>
<body${bodyspace}${bodycode}>
<div id="text">
HTML;
} /* html_begin */

// Generates HTML code for the end of a HTML page.
function html_end ()
{
	// return the code for the end of the HTML page
	return <<< HTML
</div>
<hr/>
<div id="footer"><p>Copyright&nbsp;&copy;&nbsp;2020-2021
by <a href="http://www.pawelpilarczyk.com/">Pawe&#x142; Pilarczyk</a>.
<br/>The contents of this page is published under the terms
of the <a href="http://www.gnu.org/copyleft/fdl.html">GNU Free
Documentation License</a>.
<br/>All trademarks, registered trademarks or brand names
are property of the respective holders and used in this document
for descriptive purposes only.</p></div>
</body>
</html>
HTML;
} /* html_end */

// Reads data from the form and sets the values of the variables.
// Sets 'requested' to true if there was at least one value requested.
// Sets 'correct' to true if and only if all defined values
// were in the correct format and the obligatory number
// of variables were defined.
// Appends 'error_msg' to inform about problems found, if any.
function get_values (&$values, &$requested, &$correct, &$error_msg, &$errorMarks, &$full)
{
	global $obligatory;
	global $labels;
	global $prefixes;
	global $datatypes;
	global $maxValues;
	global $minValues;

	// read the value for the "full" variable and convert it to an integer
	if (array_key_exists ("full", $_REQUEST))
	{
		$full = intval ($_REQUEST ["full"]);
	}

	// read the values into an array
	$values = array ();
	for ($i = 0; $i < count ($labels); $i ++)
	{
		$label = $labels[$i];
		if (array_key_exists ($labels[$i], $_REQUEST))
		{
			// get the requested value
			$value = $_REQUEST [$labels[$i]];
			// trim leading and trailing spaces
			$value = trim ($value);
			// append it at the end of the array of values
			array_push ($values, $value);
			// update the information that something was requested
			$requested = true;
		}
		else
			array_push ($values, "");
		array_push ($errorMarks, false);
	}

	// verify the correctness of the values
	$correct = true;
	$error_msg = "";

	// check if the obligatory nonempty values were read
	if ($requested)
	{
		for ($i = 0; $i < $obligatory; $i ++)
		{
			if ($values[$i] === '')
				$errorMarks[$i] = true;
		}
		for ($i = 0; $i < $obligatory; $i ++)
		{
			if ($values[$i] !== '')
				continue;
			$correct = false;
			if (!empty ($error_msg))
				$error_msg .= "<br/>";
			$error_msg .= "It is obligatory to provide " .
				"the first ${obligatory} values.";
			return;
		}
	}

	// make sure the numbers contain allowed characters only
	$double_pattern = '/^[-+]?\d*\.?\d+(?:e[-+]?\d*)?$/';
	$int_pattern = '/^\d+$/';
	for ($i = 0; $i < count ($datatypes); $i ++)
	{
		if (($datatypes[$i] == 0) || ($values[$i] === ''))
			continue;
		$pattern = ($datatypes[$i] == 1) ? $int_pattern :
			$double_pattern;
		if (preg_match ($pattern, $values[$i]))
			continue;
		$errorMarks[$i] = true;
		if (!empty ($error_msg))
			$error_msg .= "<br/>";
		$index = $i + 1;
		$prefix = trim ($prefixes[$i]);
		$error_msg .= "Wrong format of parameter ${index} (${prefix}).";
		$correct = false;
	}

	// make sure the values don't exceed the maximal allowed ones
	for ($i = 0; $i < count ($maxValues); $i ++)
	{
		if (empty ($maxValues[$i]) || ($values[$i] === ''))
			continue;
		$value = ($datatypes[$i] == 1) ?
			intval ($values[$i]) : floatval ($values[$i]);
		$maxValue = ($datatypes[$i] == 1) ?
			intval ($maxValues[$i]) : floatval ($maxValues[$i]);
		if ($value <= $maxValue)
			continue;
		$errorMarks[$i] = true;
		if (!empty ($error_msg))
			$error_msg .= "<br/>";
		$index = $i + 1;
		$error_msg .= "The value of parameter ${index} " .
			"(${prefixes[$i]}${values[$i]}) " .
			"exceeds the allowed maximum ${maxValues[$i]}.";
		$correct = false;
	}

	// make sure the values are not below the minimal allowed ones
	for ($i = 0; $i < count ($minValues); $i ++)
	{
		if (empty ($minValues[$i]) || ($values[$i] === ''))
			continue;
		$value = ($datatypes[$i] == 1) ?
			intval ($values[$i]) : floatval ($values[$i]);
		$minValue = ($datatypes[$i] == 1) ?
			intval ($minValues[$i]) : floatval ($minValues[$i]);
		if ($value >= $minValue)
			continue;
		$errorMarks[$i] = true;
		if (!empty ($error_msg))
			$error_msg .= "<br/>";
		$index = $i + 1;
		$error_msg .= "The value of parameter ${index} " .
			"(${prefixes[$i]}${values[$i]}) " .
			"must be at least ${minValues[$i]}.";
		$correct = false;
	}

	return;
} /* get_values */

// Creates a form to imput data.
function html_form ($values, $errorMarks, $fullForm)
{
	global $captions;
	global $labels;
	global $maxValues;
	global $minValues;
	global $prefixes;
	global $alwaysShow;
	global $run_page;

	$rows = '';
	$codeHideShow = '';
	for ($i = 0; $i < count ($captions); $i ++)
	{
		if ($minValues [$i])
			$minInfo = "(min ${minValues[$i]})";
		else
			$minInfo = "";
		if ($maxValues [$i])
			$maxInfo = "(max ${maxValues[$i]})";
		else
			$maxInfo = "";
		if ($errorMarks[$i])
		{
			$colorBegin = "<font color=\"red\">";
			$colorEnd = "</font>";
			$bgColor = " bgcolor=\"yellow\"";
		}
		else
		{
			$colorBegin = "";
			$colorEnd = "";
			$bgColor = "";
		}
		$prefix = str_replace ("-", "&#8209;", $prefixes[$i]);
		$index = $i + 1;
		$display = ($alwaysShow[$i] or $fullForm) ? "table-row" : "none";
		if (!$alwaysShow[$i])
			$codeHideShow .= "\tdocument.getElementById(\"formRow${index}\").style.display = displayNew;\n";
		$rows .= <<<ROW
<tr${bgColor} id="formRow${index}" style="display:${display};">
<td>${colorBegin}${index}. ${captions[$i]}${colorEnd}</td>
<td>&nbsp;<tt>${prefix}</tt></td>
<td><input type="text" name="${labels[$i]}" value="${values[$i]}"/></td>
<td>${minInfo} ${maxInfo}</td>
</tr>
ROW;
	}

	$showMoreTxt = "Show more parameters";
	$showLessTxt = "Show fewer parameters";
	$showMoreLess = $fullForm ? $showLessTxt : $showMoreTxt;

	return <<<FORM
<p><form name="form_input" id="form_input"
method="get" action="${run_page}"
enctype="multipart/form-data"><table>
${rows}
<script type="text/javascript">
<!--
function toggleShow()
{
	var fullForm = parseInt (document.getElementById("full").value);
	fullForm = 1 - fullForm;
	document.getElementById("full").value = fullForm.toString();
	var displayNew = fullForm ? "table-row" : "none";
${codeHideShow}
	var buttonValue = fullForm ? "${showLessTxt}" : "${showMoreTxt}";
	document.getElementById("showButton").innerHTML = buttonValue;
	//alert("Value = " + document.getElementById("full").value);
	return;
}
function runClicked()
{
	// See this web page for a list of display styles:
	// https://www.w3schools.com/cssref/pr_class_display.asp
	document.getElementById("runButton").style.display = "none";
	document.getElementById("results").style.display = "none";
	document.getElementById("running").style.display = "block";
	return;
}
//-->
</script>
<tr>
<td></td>
<td></td>
<td><input id="runButton" type="submit" value="RUN" onclick="runClicked();"/></td>
<td><input id="full" name="full" type="hidden" value="${fullForm}"/></td>
</tr>
</table></form></p>
<p><button id="showButton" onclick="toggleShow();">${showMoreLess}</button></p>
FORM;
} /* html_form */

// Creates a table with the result of the computation.
function html_result ($arguments, $result)
{
	global $program_name;

//	$result = preg_replace ("/^/m", "<br/>", $result);
//	$result = preg_replace ("/^<br\/>/", "", $result);

	return <<<RESULT
<span id="results" style="display:inline;">
<p>Command line:</p>
<p align="left"><tt>${program_name} ${arguments}</tt></p>
<p>Result:</p>
<p align="left"><pre>${result}</pre></p>
</span>
RESULT;
} /* html_result */

// Creates a paragraph with the error message.
function html_error ($error_msg)
{
	return <<<ERROR
<p>ERROR: There were some problems found:<br/>${error_msg}</p>
ERROR;
} /* html_error */

// Prepares the command-line arguments based on the values provided.
// Runs the computation and returns the output of the program.
function run_computation ($values, &$arguments)
{
	global $program_path;
	global $max_running_programs;
	global $prefixes;

	// prepare the command line
	$arguments = '';
	for ($i = 0; $i < count ($prefixes); $i ++)
	{
		if ($values[$i] === '')
			continue;
		if ($arguments !== '')
			$arguments .= ' ';
		$arguments .= $prefixes[$i] . $values[$i];
	}
	$command = "${program_path} ${arguments}";

	// refuse running the program if there are at least 2 others running
	$delay_time = rand (10, 500);
	usleep (1000 * $delay_time);
	$ps = shell_exec ("ps -A -f | grep \"[0-9] $program_path\"");
	$number_of_lines = substr_count ($ps, "\n");
	//return ($ps . "DEBUG: number of lines = " . substr_count ($ps, "\n"));
	if ($number_of_lines >= $max_running_programs)
	{
		return <<<TOOMANY
Unfortunately, it looks like there are at least $max_running_programs copies
of the program already running at this server.

* * * Please, try again later. * * *

Or - even better - you are welcome to download the source code
of the program, compile it at your own computer, and then run it
without limitations!
TOOMANY;
	}

	// run the program and return its output
	return shell_exec (escapeshellcmd($command));
} /* run_computation */

// ###################################################

// prepare the beginning and the ending of the HTML page
$htmlbeg = html_begin ($title, "", "", true);
$htmlend = html_end ();

// get the requested parameters
$values = array ();
$requested = false;
$correct = false;
$error_msg = "";
$errorMarks = array ();
$fullForm = 0;
$arguments = get_values ($values, $requested, $correct, $error_msg, $errorMarks, $fullForm);

$htmlresult = <<< RESULT
<p id="results">[After you hit the RUN button to run the program,
the results of the computation will be displayed here
after the computation has been completed.]</p>
RESULT;

// fill in the default values if no values were previously requested
if (!$requested)
{
	$values = $defaults;
}
// or run the computation if everything seems to be correct
elseif ($correct)
{
	$arguments = '';
	$result = run_computation ($values, $arguments);
	$htmlresult = html_result ($arguments, $result);
}

// prepare a message that indicates that the computations are running
$htmlrunning = <<< RUNNING
<p id="running" style="display:none;"><b>Please, wait. The computations are running...</b></p>
RUNNING;

// if there was an error in the parameters then show the message
$htmlerror = empty ($error_msg) ? "" : html_error ($error_msg);

// prepare a form with new parameters
$htmlform = html_form ($values, $errorMarks, $fullForm);

// prepare the entire HTML page
// <li><a href="preprint.pdf">preprint.pdf</a> &ndash; a preprint of the paper</li>
$html = <<< HTML
${htmlbeg}
<h1>Iterations of the Quadratic Map</h1>

<p><a href="./">This website</a> contains software and data relatd to two papers:
<ul>
<li>[1] <a href="https://arxiv.org/abs/2004.13444"
><em>Rigorous numerics for the quadratic family</em></a> by
A.&nbsp;Golmakani, C.&nbsp;E.&nbsp;Koudjinan,
<a href="https://www.stefanoluzzatto.net/">S.&nbsp;Luzzatto</a>,
and <a href="http://www.pawelpilarczyk.com/">P.&nbsp;Pilarczyk</a></li>
<li>[2] <a href="NOT-YET-AVAILABLE"><em>Rigorous computation of escape times
for parameter intervals in the quadratic map</em></a> by
<a href="http://www.pawelpilarczyk.com/">P.&nbsp;Pilarczyk</a> and
<a href="https://www.stefanoluzzatto.net/">S.&nbsp;Luzzatto</a></li>
</ul>
</p>

<p>Files for download related to the paper [1]:
<ul>
<li><a href="https://www.pawelpilarczyk.com/quadr/progcode.zip">progcode.zip</a> &ndash; the source code of the program,
published here under the terms of the
<a href="https://www.gnu.org/licenses/gpl-3.0.html">GNU
General Public License, version 3</a></li>
<li><a href="https://www.pawelpilarczyk.com/quadr/phpcode.zip">phpcode.zip</a> &ndash; the source code of the website
(programmed in PHP), published here under the terms of the
<a href="https://www.gnu.org/licenses/agpl-3.0.html">GNU
Affero General Public License, version 3</a></li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/computation_log.zip">computation_log.zip</a> (91 MB) &ndash;
the original computation log produced by the program,
which is a basis for the results discussed in the paper and for the figures contained therein
(227MB after unzipping)</li>
</ul>
</p>

<p>Files for download related to the paper [2] (see note below on the data contained in the archive files):
<ul>
<li><a href="https://www.pawelpilarczyk.com/quadr/progcode.zip">progcode.zip</a> &ndash; the source code of the program,
published here under the terms of the
<a href="https://www.gnu.org/licenses/gpl-3.0.html">GNU
General Public License, version 3</a></li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/noChoppingTest.zip">noChoppingTest.zip</a> (39 MB)
&ndash; computations used in Section 4 to check the effect of the number of bisection steps on the results</li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/w10data.zip">w10data.zip</a> (31 MB)
&ndash; data from computations for six intervals: [1.4,1.5], &hellip;, [1.9,2.0] (used for Figure 6)</li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/w4data.zip">w4data.zip</a> (0.7 MB)
&ndash; complete computation with <em>w</em>=10<sup>&minus;4</sup> (for Figures in Section 3)</li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/w5data.zip">w5data.zip</a> (5 MB)
&ndash; complete computation with <em>w</em>=10<sup>&minus;5</sup> (for Figures in Section 3)</li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/w6data.zip">w6data.zip</a> (41 MB)
&ndash; complete computation with <em>w</em>=10<sup>&minus;6</sup> (for Figures in Section 3)</li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/w7data.zip">w7data.zip</a> (273 MB)
&ndash; complete computation with <em>w</em>=10<sup>&minus;7</sup> (for Figures in Section 3)</li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/w8data_15-30.zip">w8data_15-30.zip</a> (453 MB)
&ndash; complete computation with <em>w</em>=10<sup>&minus;8</sup> and <em>N</em><sub>0</sub>=15,&hellip;,30 (for Figures in Section 3)</li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/w8data_31-35.zip">w8data_31-35.zip</a> (588 MB)
&ndash; complete computation with <em>w</em>=10<sup>&minus;8</sup> and <em>N</em><sub>0</sub>=31,&hellip;,35 (for Figures in Section 3)</li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/w8data_36-40.zip">w8data_36-40.zip</a> (620 MB)
&ndash; complete computation with <em>w</em>=10<sup>&minus;8</sup> and <em>N</em><sub>0</sub>=36,&hellip;,40 (for Figures in Section 3)</li>
<li><a href="http://ppilarczyk-wirt-s.mif.pg.gda.pl/quadr/bisectionTest.zip">bisectionTest.zip</a> (0.1 MB)
&ndash; computations used in Section 4 to check the effect of the number of bisection steps on the results</li>
</ul>
In most cases, instead of providing the complete ".log" files produced by the program,
".csv" files containing only the data used for the graphs are provided,
extracted from the ".log" files, and the original ".log" files
without the raw data are provided as ".txt" files.
Additionally, ".res" files are available that contain the information
on the resources used in each case,
measured by David MacKenzie&apos;s <em>time</em> program
available in most Linux systems.
</p>

<hr/>

<h2>Computations</h2>

<p>An interface is provided to running the program introduced in the paper [1].
Since some computations can be very lengthy and memory-consuming,
there are some limits set up on what one can do using this interface.
Please, download and compile the program to run it at your computer
in order to use full capabilities of the software.</p>

<p>Please, fill out the form with desired values of parameters,
and click the RUN button to see the result under the form.
Then you are welcome to adjust some parameters,
and run the computation again.<br/>
<em>Please, note that in the short version of the form, there are only
a few selected parameters shown below (thus the numbering is not
contiguous). Click the button below to toggle between the short list
and the long one.</em></p>

${htmlform}

<hr/>

${htmlrunning}

${htmlresult}

${htmlerror}

${htmlend}
HTML;

// output the HTML page
echo $html;

?>